package com.myoa.engineering.crawl.shopping.configuration.datasource.properties;

import com.myoa.engineering.crawl.shopping.support.util.ObjectUtil;
import lombok.Data;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.util.List;

@Component
@Data
@ConfigurationProperties(prefix = "datasource")
public class DatasourceProperties {

    private List<DataSourcePropertiesUnit> units;

    @Data
    public static class DataSourcePropertiesUnit {

        private String unitName;
        private String schemaName;
        private String connectionParameters;
        private String dbConnectionUrl;
        private Boolean isSimpleConnectionUrl;
        private String username;
        private String password;
        private String driverClassName;

        public String toCompletedJdbcUrl() {
            if (ObjectUtil.isEmpty(isSimpleConnectionUrl) || isSimpleConnectionUrl == false) {
                return String.format("%s/%s?%s", dbConnectionUrl, schemaName, connectionParameters);
            }
            return dbConnectionUrl;
        }
    }

    public DataSourcePropertiesUnit find(String unitName) {
        return units.stream()
                    .filter(e -> e.getUnitName().equals(unitName))
                    .findFirst()
                    .orElseThrow(
                            () -> new IllegalArgumentException(this.getClass().getName() + ": unitName Not found. " + unitName));
    }

}
