package com.myoa.engineering.crawl.shopping.crawlhandler;

import com.myoa.engineering.crawl.shopping.crawlhandler.parser.PpomppuArticleParserV2;
import com.myoa.engineering.crawl.shopping.domain.entity.v2.Article;
import com.myoa.engineering.crawl.shopping.infra.client.ppomppu.PpomppuBoardClientV2;
import com.myoa.engineering.crawl.shopping.service.ArticleCommandService;
import com.myoa.engineering.crawl.shopping.support.dto.constant.CrawlTarget;
import org.springframework.stereotype.Component;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Stream;

@Component
public class PpomppuCrawlDomesticHandler implements CrawlHandler {

    private final PpomppuBoardClientV2 ppomppuBoardClient;
    private final PpomppuArticleParserV2 ppomppuArticleParserV2;
    private final ArticleCommandService articleCommandService;

    public PpomppuCrawlDomesticHandler(PpomppuBoardClientV2 ppomppuBoardClient,
                                       PpomppuArticleParserV2 ppomppuArticleParserV2,
                                       ArticleCommandService articleCommandService) {
        this.ppomppuBoardClient = ppomppuBoardClient;
        this.ppomppuArticleParserV2 = ppomppuArticleParserV2;
        this.articleCommandService = articleCommandService;
    }

    @Override
    public CrawlTarget getCrawlTarget() {
        return CrawlTarget.PPOMPPU_DOMESTIC;
    }

    @Override
    public void handle() {

        String boardHtmlPage1 = ppomppuBoardClient.getBoardHtml("/zboard/zboard.php", generateRequestParams(1));
        List<Article> parsedPage1 = ppomppuArticleParserV2.parse(boardHtmlPage1);

        String boardHtmlPage2 = ppomppuBoardClient.getBoardHtml("/zboard/zboard.php", generateRequestParams(2));
        List<Article> parsedPage2 = ppomppuArticleParserV2.parse(boardHtmlPage2);

        List<Article> merged = Stream.of(parsedPage1, parsedPage2)
                                     .flatMap(List::stream)
                                     .map(e -> e.updateCrawlTarget(getCrawlTarget()))
                                     .toList();

        articleCommandService.upsert(merged);
    }

    private Map<String, String> generateRequestParams(int pageId) {
        Map<String, String> params = new HashMap<>();
        params.put("id", "ppomppu");
        params.put("page", String.valueOf(pageId));
        return params;
    }
}
