package com.myoa.engineering.crawl.shopping.service;

import com.myoa.engineering.crawl.shopping.configuration.slack.properties.SlackSecretProperties;
import com.myoa.engineering.crawl.shopping.dto.slack.SlackMessageDTO;
import com.myoa.engineering.crawl.shopping.infra.client.slack.SlackAPIClient;
import org.springframework.stereotype.Service;

@Service
public class UserNotifyService {

    private static final String SLACK_PROPERTIES_UNIT_NAME = "shopping-crawler";
//    private static final String NOTIFY_CHANNEL_ID = "notify_shopping";
//    private static final String NOTIFY_ICON_EMOJI = ":monge_big:";
//    private static final String NOTIFY_BOT_NAME = "몽이 탈호구봇";

    private final SlackAPIClient slackAPIClient;
    private final SlackSecretProperties.SlackSecretPropertiesUnit slackSecretProperties;

    public UserNotifyService(SlackAPIClient slackAPIClient,
                             SlackSecretProperties slackSecretProperties) {
        this.slackAPIClient = slackAPIClient;
        this.slackSecretProperties = slackSecretProperties.find(SLACK_PROPERTIES_UNIT_NAME);
    }

    public void notify(String message) {
        SlackMessageDTO slackMessageDTO = SlackMessageDTO.builder()
                                                         .channel(slackSecretProperties.getChannel())
                                                         .text(message)
                                                         .iconEmoji(slackSecretProperties.getIconEmoji())
                                                         .username(slackSecretProperties.getUsername())
                                                         .build();
        slackAPIClient.sendMessage(slackMessageDTO, slackSecretProperties.getToken());
    }
}
