package com.myoa.engineering.crawl.shopping.util;

import com.fasterxml.jackson.annotation.JsonInclude;
import com.fasterxml.jackson.core.json.JsonReadFeature;
import com.fasterxml.jackson.databind.DeserializationFeature;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.fasterxml.jackson.databind.SerializationFeature;
import com.fasterxml.jackson.databind.json.JsonMapper;
import com.fasterxml.jackson.datatype.jdk8.Jdk8Module;
import com.fasterxml.jackson.datatype.jsr310.JavaTimeModule;
import com.fasterxml.jackson.module.paramnames.ParameterNamesModule;

public final class ObjectMapperFactory {

    public static final ObjectMapper DEFAULT_MAPPER;
    public static final ObjectMapper REDIS_MAPPER;
    @Deprecated
    public static final ObjectMapper LOGGING_MAPPER;

    static {
        DEFAULT_MAPPER = initDefaultMapper();
        REDIS_MAPPER = initRedisMapper();
        LOGGING_MAPPER = JsonMapper.builder()
                                   .serializationInclusion(JsonInclude.Include.NON_EMPTY)
                                   .build();
    }

    private ObjectMapperFactory() {
    }

    private static ObjectMapper initDefaultMapper() {
        return JsonMapper.builder()
                         .configure(JsonReadFeature.ALLOW_UNQUOTED_FIELD_NAMES.mappedFeature(), true)
                         .configure(JsonReadFeature.ALLOW_UNESCAPED_CONTROL_CHARS.mappedFeature(), true)
                         .disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES)
                         .disable(SerializationFeature.FAIL_ON_EMPTY_BEANS)
                         .disable(SerializationFeature.FAIL_ON_UNWRAPPED_TYPE_IDENTIFIERS)
                         .enable(DeserializationFeature.READ_UNKNOWN_ENUM_VALUES_AS_NULL)
                         .disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS)
                         .serializationInclusion(JsonInclude.Include.NON_NULL)
                         .build()
                         .registerModule(new ParameterNamesModule())
                         .registerModule(new Jdk8Module())
                         .registerModule(new JavaTimeModule());
    }

    private static ObjectMapper initRedisMapper() {
        return JsonMapper.builder()
                         .configure(JsonReadFeature.ALLOW_UNQUOTED_FIELD_NAMES.mappedFeature(), true)
                         .configure(JsonReadFeature.ALLOW_UNESCAPED_CONTROL_CHARS.mappedFeature(), true)
                         .disable(DeserializationFeature.FAIL_ON_UNKNOWN_PROPERTIES)
                         .disable(SerializationFeature.FAIL_ON_EMPTY_BEANS)
                         .disable(SerializationFeature.FAIL_ON_UNWRAPPED_TYPE_IDENTIFIERS)
                         .enable(DeserializationFeature.READ_UNKNOWN_ENUM_VALUES_AS_NULL)
                         .disable(SerializationFeature.WRITE_DATES_AS_TIMESTAMPS)
                         .serializationInclusion(JsonInclude.Include.NON_NULL)
                         .build()
                         .registerModule(new ParameterNamesModule())
                         .registerModule(new Jdk8Module())
                         .registerModule(new JavaTimeModule());
    }
}
