package com.myoa.engineering.crawl.shopping.event.handler;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;
import org.ahocorasick.trie.Emit;
import org.ahocorasick.trie.Trie;
import org.jeasy.random.EasyRandom;
import org.jeasy.random.EasyRandomParameters;
import org.junit.jupiter.api.Test;

import java.util.Collection;
import java.util.List;

class ArticleUpsertEventListenerTest {

    @Test
    public void test1() {

        List<String> baseData = List.of(
                "[공홈]베베숲 시그니처 위드 블루 20팩 (25,990원/무료)2 [기타]",
                "[현대H몰]10주년 스페셜 에디션 봉고데기 40mm 세트 (67,640원/무료)3 [가전/가구]",
                "[SSG]필립스 면도기 칫솔 기획전( 437,000원~/무료)1 [기타]",
                "(티몬)제스프리 골드키위 중대과 1.8kg내외 (카페,토페 14,823원/무배)3 [식품/건강]",
                "[공식몰]연세생활건강 당뇨영양식 24팩 (33,500원/유배)1 [식품/건강]",
                "[인터파크]크리스탈라이트 아이스티 에이드 온더고 60개입 멀티팩(17,800원/무료)6 [기타]",
                "[인팍쇼핑]샤카웨어 액티브 맥스 헤비웨이트 10종 택 1 (7,350원/무료)7 [의류/잡화]"
        );

        Trie trie = Trie.builder()
                        .ignoreCase()
                        .addKeyword("블루")
                        .addKeyword("봉고")
                        .build();

        List<Collection<Emit>> list = baseData.stream()
                                              .map(trie::parseText)
                                              .toList();

        System.out.println(list);
    }

    @Test
    void test2() throws InterruptedException {
        EasyRandomParameters paramsKeyword = new EasyRandomParameters()
                .stringLengthRange(2, 10);
        EasyRandom GEN_KEYWORD = new EasyRandom(paramsKeyword);

        EasyRandomParameters paramsSentence = new EasyRandomParameters()
                .stringLengthRange(100, 1000);
        EasyRandom GEN_SENTENCE = new EasyRandom(paramsSentence);

        Trie.TrieBuilder trieBuilder = Trie.builder().ignoreOverlaps();

        GEN_KEYWORD.objects(TestA.class, 1000)
                   .map(TestA::getFieldA)
                   .forEach(trieBuilder::addKeyword);
        Trie trie = trieBuilder.build();

        List<Collection<Emit>> listOfEmits = GEN_SENTENCE.objects(TestA.class, 2000)
                                                  .map(TestA::getFieldA)
                                                  .map(trie::parseText)
                                                  .toList();

        for(Collection<Emit> emits : listOfEmits) {
            System.out.println(emits);
            System.out.println("--------------------------");
        }

    }


    @Data
    @AllArgsConstructor
    @NoArgsConstructor
    public class TestA {
        private String fieldA;

    }

}