package com.myoa.engineering.crawl.shopping.util;

import com.fasterxml.jackson.core.JsonProcessingException;
import com.fasterxml.jackson.databind.ObjectMapper;
import org.springframework.lang.NonNull;

import java.io.IOException;
import java.util.Arrays;
import java.util.List;
import java.util.Scanner;
import java.util.stream.Collectors;

public final class TestDataUtils {

    private static final ObjectMapper OBJECT_MAPPER;

    static {
        OBJECT_MAPPER = ObjectMapperFactory.DEFAULT_MAPPER;
    }

    private TestDataUtils() {
    }

    public static <T> T inputStreamToObject(@NonNull String resourcePath,
                                            @NonNull Class<T> clazz) throws IOException {
        return OBJECT_MAPPER.readValue(fileToString(resourcePath), clazz);
    }

    public static <T> List<T> inputStreamsToList(@NonNull Class<T> clazz,
                                                 @NonNull String... resourcePath) throws IOException {
        return inputStreamsToList(Arrays.asList(resourcePath), clazz);
    }

    public static <T> List<T> inputStreamsToList(@NonNull List<String> resourcePath,
                                                 @NonNull Class<T> clazz) throws IOException {
        return resourcePath.stream()
                           .map(e -> {
                               try {
                                   return OBJECT_MAPPER.readValue(fileToString(e), clazz);
                               } catch (JsonProcessingException jsonProcessingException) {
                                   jsonProcessingException.printStackTrace();
                                   return null;
                               }
                           })
                           .collect(Collectors.toList());
    }

    public static <T> List<T> inputStreamToList(@NonNull String resourcePath,
                                                @NonNull Class<T> clazz) throws IOException {
        return (List<T>) OBJECT_MAPPER.readValue(
                fileToString(resourcePath),
                OBJECT_MAPPER.getTypeFactory().constructCollectionType(List.class, clazz));
    }

    public static String fileToString(@NonNull String resourcePath) {

        final ClassLoader classLoader = Thread.currentThread().getContextClassLoader();
        final Scanner s = new Scanner(classLoader.getResourceAsStream(resourcePath)).useDelimiter("\\A");
        return s.hasNext() ? s.next() : "";
    }

}
